@@ -8,9 +8,16 @@
 //! <i>This documentation has been built with the following Cargo features:
 // Make sure the commas are inserted at the right places!
 #![cfg_attr(feature = "asp3", doc = "`asp3`")]
+#![cfg_attr(feature = "fmp3", doc = "`fmp3`")]
+#![cfg_attr(feature = "solid_asp3", doc = "`solid_asp3`")]
+#![cfg_attr(feature = "solid_fmp3", doc = "`solid_fmp3`")]
 #![cfg_attr(feature = "none", doc = "`none`")]
 #![cfg_attr(feature = "dcre", doc = ", `dcre`")]
 #![cfg_attr(feature = "rstr_task", doc = ", `rstr_task`")]
+#![cfg_attr(feature = "messagebuf", doc = ", `messagebuf`")]
+#![cfg_attr(feature = "ovrhdr", doc = ", `ovrhdr`")]
+#![cfg_attr(feature = "subprio", doc = ", `subprio`")]
+#![cfg_attr(feature = "pi_mutex", doc = ", `pi_mutex`")]
 //! </i>
 #![allow(non_camel_case_types)]
 #![allow(unused_imports)]
