@@ -7,6 +7,16 @@ use crate::{
     time::{Duration, Timeout},
 };
 
+// TODO: sta_ovr
+// TODO: stp_ovr
+// TODO: ref_ovr
+// TODO: chg_spr
+// TODO: mact_tsk
+// TODO: chg_spr
+// TODO: TA_ACT
+// TODO: TA_NOACTQUE
+// TODO: TA_RTSK
+
 define_error_kind! {
     /// Error type for [`TaskRef::activate`].
     pub enum ActivateError {
@@ -37,6 +47,50 @@ impl ErrorKind for ActivateError {
     }
 }
 
+define_error_kind! {
+    /// Error type for [`TaskRef::activate_on`].
+    #[cfg(any(feature = "none", feature = "fmp3", feature = "solid_fmp3"))]
+    #[cfg_attr(feature = "doc_cfg", doc(cfg(any(feature = "none", feature = "fmp3", feature = "solid_fmp3"))))]
+    pub enum ActivateOnError {
+        #[cfg(not(feature = "none"))]
+        BadContext,
+        #[cfg(not(feature = "none"))]
+        BadId,
+        /// The task is a restricted task.
+        #[cfg(all(not(feature = "none"), feature = "rstr_task"))]
+        NotSupported,
+        #[cfg(any())]
+        AccessDenied,
+        #[cfg(not(feature = "none"))]
+        QueueOverflow,
+        /// The class the task belongs to does not permit assigning tasks to the
+        /// specified processor.
+        #[cfg(not(feature = "none"))]
+        BadParam,
+    }
+}
+
+#[cfg(any(feature = "none", feature = "fmp3", feature = "solid_fmp3"))]
+impl ErrorKind for ActivateOnError {
+    fn from_error_code(code: ErrorCode) -> Option<Self> {
+        match code.get() {
+            #[cfg(not(feature = "none"))]
+            abi::E_CTX => Some(Self::BadContext(Kind::from_error_code(code))),
+            #[cfg(not(feature = "none"))]
+            abi::E_ID | abi::E_NOEXS => Some(Self::BadId(Kind::from_error_code(code))),
+            #[cfg(all(not(feature = "none"), feature = "rstr_task"))]
+            abi::E_NOSPT => Some(Self::NotSupported(Kind::from_error_code(code))),
+            #[cfg(any())]
+            abi::E_OACV => Some(Self::AccessDenied(Kind::from_error_code(code))),
+            #[cfg(not(feature = "none"))]
+            abi::E_QOVR => Some(Self::QueueOverflow(Kind::from_error_code(code))),
+            #[cfg(not(feature = "none"))]
+            abi::E_PAR => Some(Self::BadParam(Kind::from_error_code(code))),
+            _ => None,
+        }
+    }
+}
+
 define_error_kind! {
     /// Error type for [`TaskRef::cancel_activate_all`].
     pub enum CancelActivateAllError {
@@ -446,6 +500,8 @@ define_error_kind! {
         /// Bad state.
         ///
         ///  - The task is dormant.
+        ///  - The task is assigned to a processor that is different from the
+        ///    current one (`E_OBJ`, NGKI3481).
         ///
         #[cfg(not(feature = "none"))]
         BadState,
@@ -488,6 +544,8 @@ define_error_kind! {
         /// Bad state.
         ///
         ///  - The task is dormant.
+        ///  - The task is assigned to a processor that is different from the
+        ///    current one (`E_OBJ`, NGKI3481).
         ///
         #[cfg(not(feature = "none"))]
         BadState,
@@ -734,6 +792,10 @@ define_error_kind! {
 #[cfg(feature = "dcre")]
 impl ErrorKind for BuildError {
     fn from_error_code(code: ErrorCode) -> Option<Self> {
+        // `Builder::build` uses `get_pid` to get the current processor and
+        // coalesces its error code into this error kind type. Thus, this error
+        // kind type must be able to handle errors from both `acre_tsk` and
+        // `get_pid`!
         match code.get() {
             // `E_MACV` is considered critical, hence excluded
             #[cfg(not(feature = "none"))]
@@ -821,6 +883,8 @@ impl Info {
     // TODO: wupcnt
     // TODO: raster
     // TODO: dister
+    // TODO: prcid
+    // TODO: actprc
 }
 
 /// `slp_tsk`: Put the current task to sleep.
@@ -996,7 +1060,7 @@ impl TaskRef<'_> {
         self.id.get()
     }
 
-    /// Get the raw object ID as [` abi::NonNullID`].
+    /// Get the raw object ID as [`abi::NonNullID`].
     #[inline]
     pub const fn as_raw_nonnull(self) -> abi::NonNullID {
         self.id
@@ -1020,6 +1084,29 @@ impl TaskRef<'_> {
         }
     }
 
+    /// `mact_tsk`: Pend an activation request for the task, assigning it to
+    /// the specified processor.
+    #[inline]
+    #[doc(alias = "mact_tsk")]
+    #[cfg(any(feature = "none", feature = "fmp3", feature = "solid_fmp3"))]
+    #[cfg_attr(
+        feature = "doc_cfg",
+        doc(cfg(any(feature = "none", feature = "fmp3", feature = "solid_fmp3")))
+    )]
+    pub fn activate_on(
+        self,
+        processor: crate::processor::Processor,
+    ) -> Result<(), Error<ActivateOnError>> {
+        match () {
+            #[cfg(not(feature = "none"))]
+            () => unsafe {
+                Error::err_if_negative(abi::mact_tsk(self.as_raw(), processor.as_raw()))?;
+                Ok(())
+            },
+            #[cfg(feature = "none")]
+            () => unimplemented!(),
+        }
+    }
     /// `can_act`: Cancel any pending activation requests for the task.
     /// Returns the number of cancelled requests.
     #[inline]
@@ -1068,6 +1155,10 @@ impl TaskRef<'_> {
     }
 
     /// `del_tsk`: Delete the task.
+    ///
+    /// # Safety
+    ///
+    /// See [Object ID Wrappers](crate#object-id-wrappers).
     #[inline]
     #[doc(alias = "del_tsk")]
     #[cfg(feature = "dcre")]
@@ -1293,7 +1384,7 @@ impl Current {
         self.inner.as_raw()
     }
 
-    /// Get the raw object ID as [` abi::NonNullID`].
+    /// Get the raw object ID as [`abi::NonNullID`].
     #[inline]
     pub const fn as_raw_nonnull(&self) -> abi::NonNullID {
         self.inner.as_raw_nonnull()
@@ -1317,14 +1408,80 @@ pub use self::owned::*;
 mod owned {
     use super::*;
 
+    #[cfg(any(feature = "none", feature = "solid_fmp3"))]
+    pub use self::processor_set::*;
+
+    #[cfg(any(feature = "none", feature = "solid_fmp3"))]
+    mod processor_set {
+        use crate::{abi, processor::Processor};
+        use core::convert::TryFrom;
+
+        /// The trait implemented by types that can be passed to
+        /// [`crate::task::Builder::processor_affinity`]. This trait is [sealed].
+        ///
+        /// [sealed]: https://rust-lang.github.io/api-guidelines/future-proofing.html#sealed-traits-protect-against-downstream-implementations-c-sealed
+        #[cfg_attr(feature = "doc_cfg", doc(cfg(feature = "dcre")))]
+        pub trait IntoProcessorSet: private::Sealed + Sized {
+            #[doc(hidden)]
+            fn into_uint_t(self) -> abi::uint_t;
+        }
+
+        /// Implements [the sealed trait pattern (C-SEALED)].
+        ///
+        /// [the sealed trait pattern (C-SEALED)]: https://rust-lang.github.io/api-guidelines/future-proofing.html#sealed-traits-protect-against-downstream-implementations-c-sealed
+        mod private {
+            use super::*;
+
+            pub trait Sealed {}
+
+            impl<T: IntoIterator<Item = Processor>> Sealed for T {}
+            impl Sealed for Processor {}
+            impl Sealed for AllProcessors {}
+        }
+
+        impl<T: IntoIterator<Item = Processor>> IntoProcessorSet for T {
+            #[doc(hidden)]
+            #[inline]
+            fn into_uint_t(self) -> abi::uint_t {
+                self.into_iter()
+                    .fold(0, |st, processor| st | processor.into_uint_t())
+            }
+        }
+
+        impl IntoProcessorSet for Processor {
+            #[doc(hidden)]
+            #[inline]
+            fn into_uint_t(self) -> abi::uint_t {
+                u32::try_from(self.as_raw() - 1)
+                    .ok()
+                    .and_then(|i| (1 as abi::uint_t).checked_shl(i))
+                    .expect("invalid processor ID")
+            }
+        }
+
+        /// An instance of [`IntoProcessorSet`] specifying all processors.
+        #[derive(Debug, Default, Clone, Copy, PartialEq, Eq)]
+        pub struct AllProcessors;
+
+        impl IntoProcessorSet for AllProcessors {
+            #[doc(hidden)]
+            #[inline]
+            fn into_uint_t(self) -> abi::uint_t {
+                abi::uint_t::MAX
+            }
+        }
+    }
+
     /// The builder type for [tasks](Task). Created by [`Task::build`].
     ///
     /// Its generic parameters are an implementation detail.
     #[cfg_attr(feature = "doc_cfg", doc(cfg(feature = "dcre")))]
+    #[must_use = "`Builder` creates nothing unless you call `.finish()`"]
     pub struct Builder<Start, Stack, InitialPriority> {
         start: Start,
         stack: Stack,
         initial_priority: InitialPriority,
+        assign_to_current_procesor: bool,
         #[cfg(not(feature = "none"))]
         raw: abi::T_CTSK,
     }
@@ -1371,7 +1528,8 @@ mod owned {
                 start: builder_hole::__start_is_not_specified__,
                 stack: builder_hole::__stack_is_not_specified__,
                 initial_priority: builder_hole::__initial_priority_is_not_specified__,
-                #[cfg(not(feature = "none"))]
+                assign_to_current_procesor: true,
+                #[cfg(any(feature = "asp3", feature = "solid_asp3"))]
                 raw: abi::T_CTSK {
                     tskatr: abi::TA_NULL,
                     exinf: abi::EXINF::uninit(),
@@ -1380,6 +1538,17 @@ mod owned {
                     stksz: 0,
                     stk: core::ptr::null_mut(),
                 },
+                #[cfg(feature = "solid_fmp3")]
+                raw: abi::T_CTSK {
+                    tskatr: abi::TA_NULL,
+                    exinf: abi::EXINF::uninit(),
+                    task: None,
+                    itskpri: 0,
+                    stksz: 0,
+                    stk: core::ptr::null_mut(),
+                    affinity: abi::uint_t::MAX,
+                    iprcid: 0,
+                },
             }
         }
     }
@@ -1393,9 +1562,12 @@ mod owned {
         ) -> Builder<(), Stack, InitialPriority> {
             let (task, exinf) = value.into_closure();
             Builder {
+                // FIXME: Use the struct update syntax when rust-lang/rfcs#2528
+                //        is implemented
                 start: (),
                 stack: self.stack,
                 initial_priority: self.initial_priority,
+                assign_to_current_procesor: self.assign_to_current_procesor,
                 #[cfg(not(feature = "none"))]
                 raw: abi::T_CTSK {
                     task: Some(task),
@@ -1413,6 +1585,7 @@ mod owned {
                 start: self.start,
                 stack: (),
                 initial_priority: self.initial_priority,
+                assign_to_current_procesor: self.assign_to_current_procesor,
                 #[cfg(not(feature = "none"))]
                 raw: abi::T_CTSK {
                     stksz: size,
@@ -1430,6 +1603,7 @@ mod owned {
                 start: self.start,
                 stack: self.stack,
                 initial_priority: (),
+                assign_to_current_procesor: self.assign_to_current_procesor,
                 #[cfg(not(feature = "none"))]
                 raw: abi::T_CTSK {
                     itskpri: value,
@@ -1437,11 +1611,76 @@ mod owned {
                 },
             }
         }
+
+        /// Specify the task's initial assigned processor. Defaults to the
+        /// current processor when unspecified.
+        #[inline]
+        #[cfg(any(feature = "none", feature = "solid_fmp3"))]
+        #[cfg_attr(
+            feature = "doc_cfg",
+            doc(cfg(any(feature = "none", feature = "solid_fmp3")))
+        )]
+        pub fn initial_processor(self, value: crate::processor::Processor) -> Self {
+            Builder {
+                assign_to_current_procesor: false,
+                #[cfg(not(feature = "none"))]
+                raw: abi::T_CTSK {
+                    iprcid: value.as_raw(),
+                    ..self.raw
+                },
+                ..self
+            }
+        }
+
+        /// Specify the task's assignable processsor set. Defaults to all
+        /// processors when unspecified.
+        ///
+        /// This function might panic if an invalid processor ID is specified.
+        ///
+        /// # Examples
+        ///
+        /// ```rust,no_run
+        /// #![feature(const_option)]
+        /// use itron::{task::Task, processor::Processor};
+        /// const P1: Processor = Processor::from_raw(1).unwrap();
+        /// const P3: Processor = Processor::from_raw(3).unwrap();
+        /// const P4: Processor = Processor::from_raw(4).unwrap();
+        /// let task = Task::build()
+        ///     .start(move || {})
+        ///     .stack(4096)
+        ///     .initial_priority(4)
+        ///     .initial_processor(P3)
+        ///     .processor_affinity([P1, P3, P4])
+        ///     .finish()
+        ///     .expect("failed to create a task");
+        /// ```
+        #[inline]
+        #[cfg(any(feature = "none", feature = "solid_fmp3"))]
+        #[cfg_attr(
+            feature = "doc_cfg",
+            doc(cfg(any(feature = "none", feature = "solid_fmp3")))
+        )]
+        pub fn processor_affinity(self, value: impl IntoProcessorSet) -> Self {
+            Builder {
+                #[cfg(not(feature = "none"))]
+                raw: abi::T_CTSK {
+                    affinity: value.into_uint_t(),
+                    ..self.raw
+                },
+                ..self
+            }
+        }
     }
 
     impl Builder<(), (), ()> {
         /// Create a task using the specified parameters.
-        pub fn finish(self) -> Result<Task, Error<BuildError>> {
+        #[allow(unused_mut)]
+        pub fn finish(mut self) -> Result<Task, Error<BuildError>> {
+            #[cfg(feature = "solid_fmp3")]
+            if self.assign_to_current_procesor {
+                unsafe { Error::err_if_negative(abi::get_pid(&mut self.raw.iprcid))? };
+            }
+
             match () {
                 #[cfg(not(feature = "none"))]
                 () => unsafe {
@@ -1504,7 +1743,7 @@ mod owned {
             self.0.as_raw()
         }
 
-        /// Get the raw object ID as [` abi::NonNullID`].
+        /// Get the raw object ID as [`abi::NonNullID`].
         #[inline]
         pub const fn as_raw_nonnull(&self) -> abi::NonNullID {
             self.0.as_raw_nonnull()
